/**
 * Modulate a value between two ranges.
 * @param value
 * @param a from [low, high]
 * @param b to [low, high]
 * @param clamp
 */
function mod(value, a, b, clamp) {
  if (clamp === void 0) {
    clamp = false;
  }

  var _ref = b[0] < b[1] ? [b[0], b[1]] : [b[1], b[0]],
      low = _ref[0],
      high = _ref[1];

  var result = b[0] + (value - a[0]) / (a[1] - b[0]) * (b[1] - b[0]);

  if (clamp) {
    if (result < low) return low;
    if (result > high) return high;
  }

  return result;
}
/**
 * Rotate a point around a center.
 * @param x The x-axis coordinate of the point.
 * @param y The y-axis coordinate of the point.
 * @param cx The x-axis coordinate of the point to rotate round.
 * @param cy The y-axis coordinate of the point to rotate round.
 * @param angle The distance (in radians) to rotate.
 */

function rotatePoint(x, y, cx, cy, angle) {
  var s = Math.sin(angle);
  var c = Math.cos(angle);
  var px = x - cx;
  var py = y - cy;
  var nx = px * c - py * s;
  var ny = px * s + py * c;
  return [nx + cx, ny + cy];
}
/**
 * Get the distance between two points.
 * @param x0 The x-axis coordinate of the first point.
 * @param y0 The y-axis coordinate of the first point.
 * @param x1 The x-axis coordinate of the second point.
 * @param y1 The y-axis coordinate of the second point.
 */

function getDistance(x0, y0, x1, y1) {
  return Math.hypot(y1 - y0, x1 - x0);
}
/**
 * Get an angle (radians) between two points.
 * @param x0 The x-axis coordinate of the first point.
 * @param y0 The y-axis coordinate of the first point.
 * @param x1 The x-axis coordinate of the second point.
 * @param y1 The y-axis coordinate of the second point.
 */

function getAngle(x0, y0, x1, y1) {
  return Math.atan2(y1 - y0, x1 - x0);
}
/**
 * Move a point in an angle by a distance.
 * @param x0
 * @param y0
 * @param a angle (radians)
 * @param d distance
 */

function projectPoint(x0, y0, a, d) {
  return [Math.cos(a) * d + x0, Math.sin(a) * d + y0];
}
/**
 * Get a point between two points.
 * @param x0 The x-axis coordinate of the first point.
 * @param y0 The y-axis coordinate of the first point.
 * @param x1 The x-axis coordinate of the second point.
 * @param y1 The y-axis coordinate of the second point.
 * @param d Normalized
 */

function getPointBetween(x0, y0, x1, y1, d) {
  if (d === void 0) {
    d = 0.5;
  }

  return [x0 + (x1 - x0) * d, y0 + (y1 - y0) * d];
}
/**
 * Get the sector of an angle (e.g. quadrant, octant)
 * @param a The angle to check.
 * @param s The number of sectors to check.
 */

function getSector(a, s) {
  if (s === void 0) {
    s = 8;
  }

  return Math.floor(s * (0.5 + a / (Math.PI * 2) % s));
}
/**
 * Get a normal value representing how close two points are from being at a 45 degree angle.
 * @param x0 The x-axis coordinate of the first point.
 * @param y0 The y-axis coordinate of the first point.
 * @param x1 The x-axis coordinate of the second point.
 * @param y1 The y-axis coordinate of the second point.
 */

function getAngliness(x0, y0, x1, y1) {
  return Math.abs((x1 - x0) / 2 / ((y1 - y0) / 2));
}
/**
 * Check whether two rectangles will collide (overlap).
 * @param x0 The x-axis coordinate of the first rectangle.
 * @param y0 The y-axis coordinate of the first rectangle.
 * @param w0 The width of the first rectangle.
 * @param h0 The height of the first rectangle.
 * @param x1 The x-axis coordinate of the second rectangle.
 * @param y1 The y-axis coordinate of the second rectangle.
 * @param w1 The width of the second rectangle.
 * @param h1 The height of the second rectangle.
 */

function doRectanglesCollide(x0, y0, w0, h0, x1, y1, w1, h1) {
  return !(x0 >= x1 + w1 || x1 >= x0 + w0 || y0 >= y1 + h1 || y1 >= y0 + h0);
}
/**
 * Find the point(s) where a segment intersects a rectangle.
 * @param x0 The x-axis coordinate of the segment's starting point.
 * @param y0 The y-axis coordinate of the segment's starting point.
 * @param x1 The x-axis coordinate of the segment's ending point.
 * @param y1 The y-axis coordinate of the segment's ending point.
 * @param x The x-axis coordinate of the rectangle.
 * @param y The y-axis coordinate of the rectangle.
 * @param w The width of the rectangle.
 * @param h The height of the rectangle.
 */

function getSegmentRectangleIntersectionPoints(x0, y0, x1, y1, x, y, w, h) {
  var points = [];

  for (var _i2 = 0, _arr2 = [[x, y, x + w, y], [x + w, y, x + w, y + h], [x + w, y + h, x, y + h], [x, y + h, x, y]]; _i2 < _arr2.length; _i2++) {
    var _arr2$_i = _arr2[_i2],
        px0 = _arr2$_i[0],
        py0 = _arr2$_i[1],
        px1 = _arr2$_i[2],
        py1 = _arr2$_i[3];
    var ints = getSegmentSegmentIntersection(px0, py0, px1, py1, x0, y0, x1, y1);

    if (ints) {
      points.push(ints);
    }
  }

  return points;
}
/**
 * Find the point, if any, where two segments intersect.
 * @param x0 The x-axis coordinate of the first segment's starting point.
 * @param y0 The y-axis coordinate of the first segment's starting point.
 * @param x1 The x-axis coordinate of the first segment's ending point.
 * @param y1 The y-axis coordinate of the first segment's ending point.
 * @param x2 The x-axis coordinate of the second segment's starting point.
 * @param y2 The y-axis coordinate of the second segment's starting point.
 * @param x3 The x-axis coordinate of the second segment's ending point.
 * @param y3 The y-axis coordinate of the second segment's ending point.
 */

function getSegmentSegmentIntersection(x0, y0, x1, y1, x2, y2, x3, y3) {
  var denom = (y3 - y2) * (x1 - x0) - (x3 - x2) * (y1 - y0);
  var numeA = (x3 - x2) * (y0 - y2) - (y3 - y2) * (x0 - x2);
  var numeB = (x1 - x0) * (y0 - y2) - (y1 - y0) * (x0 - x2);

  if (denom === 0) {
    if (numeA === 0 && numeB === 0) {
      return undefined; // Colinear
    }

    return undefined; // Parallel
  }

  var uA = numeA / denom;
  var uB = numeB / denom;

  if (uA >= 0 && uA <= 1 && uB >= 0 && uB <= 1) {
    return [x0 + uA * (x1 - x0), y0 + uA * (y1 - y0)];
  }

  return undefined; // No intersection
}
/**
 * Get the intersection points between a line segment and a rectangle with rounded corners.
 * @param x0 The x-axis coordinate of the segment's starting point.
 * @param y0 The y-axis coordinate of ththe segment's ending point.
 * @param x1 The delta-x of the ray.
 * @param y1 The delta-y of the ray.
 * @param x The x-axis coordinate of the rectangle.
 * @param y The y-axis coordinate of the rectangle.
 * @param w The width of the rectangle.
 * @param h The height of the rectangle.
 * @param r The corner radius of the rectangle.
 */

function getSegmentRoundedRectangleIntersectionPoints(x0, y0, x1, y1, x, y, w, h, r) {
  var mx = x + w,
      my = y + h,
      rx = x + r,
      ry = y + r,
      mrx = x + w - r,
      mry = y + h - r;
  var segments = [[x, mry, x, ry, x, y], [rx, y, mrx, y, mx, y], [mx, ry, mx, mry, mx, my], [mrx, my, rx, my, x, my]];
  var corners = [[rx, ry, Math.PI, Math.PI * 1.5], [mrx, ry, Math.PI * 1.5, Math.PI * 2], [mrx, mry, 0, Math.PI * 0.5], [rx, mry, Math.PI * 0.5, Math.PI]];
  var points = [];
  segments.forEach(function (segment, i) {
    var px0 = segment[0],
        py0 = segment[1],
        px1 = segment[2],
        py1 = segment[3];
    var _corners$i = corners[i],
        cx = _corners$i[0],
        cy = _corners$i[1],
        as = _corners$i[2],
        ae = _corners$i[3];
    getSegmentCircleIntersections(cx, cy, r, x0, y0, x1, y1).filter(function (pt) {
      var pointAngle = normalizeAngle(getAngle(cx, cy, pt[0], pt[1]));
      return pointAngle > as && pointAngle < ae;
    }).forEach(function (pt) {
      return points.push(pt);
    });
    var segmentInt = getSegmentSegmentIntersection(x0, y0, x1, y1, px0, py0, px1, py1);

    if (!!segmentInt) {
      points.push(segmentInt);
    }
  });
  return points;
}
/**
 * Get the point(s) where a line segment intersects a circle.
 * @param cx The x-axis coordinate of the circle's center.
 * @param cy The y-axis coordinate of the circle's center.
 * @param r The circle's radius.
 * @param x0 The x-axis coordinate of the segment's starting point.
 * @param y0 The y-axis coordinate of ththe segment's ending point.
 * @param x1 The delta-x of the ray.
 * @param y1 The delta-y of the ray.
 */

function getSegmentCircleIntersections(cx, cy, r, x0, y0, x1, y1) {
  var b,
      c,
      d,
      u1,
      u2,
      ret,
      retP1,
      retP2,
      v1 = [x1 - x0, y1 - y0],
      v2 = [x0 - cx, y0 - cy];
  b = v1[0] * v2[0] + v1[1] * v2[1];
  c = 2 * (v1[0] * v1[0] + v1[1] * v1[1]);
  b *= -2;
  d = Math.sqrt(b * b - 2 * c * (v2[0] * v2[0] + v2[1] * v2[1] - r * r));

  if (isNaN(d)) {
    // no intercept
    return [];
  }

  u1 = (b - d) / c; // these represent the unit distance of point one and two on the line

  u2 = (b + d) / c;
  retP1 = []; // return points

  retP2 = [];
  ret = []; // return array

  if (u1 <= 1 && u1 >= 0) {
    // add point if on the line segment
    retP1[0] = x0 + v1[0] * u1;
    retP1[1] = y0 + v1[1] * u1;
    ret[0] = retP1;
  }

  if (u2 <= 1 && u2 >= 0) {
    // second add point if on the line segment
    retP2[0] = x0 + v1[0] * u2;
    retP2[1] = y0 + v1[1] * u2;
    ret[ret.length] = retP2;
  }

  return ret;
}
/**
 * Normalize an angle (in radians)
 * @param radians The radians quantity to normalize.
 */

function normalizeAngle(radians) {
  return radians - Math.PI * 2 * Math.floor(radians / (Math.PI * 2));
}
/**
 *
 * @param x The x-axis coordinate of the ray's origin.
 * @param y The y-axis coordinate of the ray's origin.
 * @param w
 * @param h
 * @param x0
 * @param y0
 * @param x1
 * @param y1
 */

function getRayRectangleIntersectionPoints(ox, oy, dx, dy, x, y, w, h) {
  var points = [];

  for (var _i3 = 0, _arr3 = [[x, y, x + w, y], [x + w, y, x + w, y + h], [x + w, y + h, x, y + h], [x, y + h, x, y]]; _i3 < _arr3.length; _i3++) {
    var _arr3$_i = _arr3[_i3],
        px0 = _arr3$_i[0],
        py0 = _arr3$_i[1],
        px1 = _arr3$_i[2],
        py1 = _arr3$_i[3];
    var ints = getRaySegmentIntersection(ox, oy, dx, dy, px0, py0, px1, py1);

    if (ints) {
      points.push(ints);
    }
  }

  return points;
}
/**
 * Get the point at which a ray intersects a segment.
 * @param x The x-axis coordinate of the ray's origin.
 * @param y The y-axis coordinate of the ray's origin.
 * @param dx The x-axis delta of the angle.
 * @param dy The y-axis delta of the angle.
 * @param x0 The x-axis coordinate of the segment's start point.
 * @param y0 The y-axis coordinate of the segment's start point.
 * @param x1 The x-axis coordinate of the segment's end point.
 * @param y1 The y-axis coordinate of the segment's end point.
 */

function getRaySegmentIntersection(x, y, dx, dy, x0, y0, x1, y1) {
  var r, s, d;

  if (dy * (x1 - x0) !== dx * (y1 - y0)) {
    d = dx * (y1 - y0) - dy * (x1 - x0);

    if (d !== 0) {
      r = ((y - y0) * (x1 - x0) - (x - x0) * (y1 - y0)) / d;
      s = ((y - y0) * dx - (x - x0) * dy) / d;

      if (r >= 0 && s >= 0 && s <= 1) {
        return [x + r * dx, y + r * dy];
      }
    }
  }

  return undefined;
}
/**
 * Get the normalized delta (x and y) for an angle.
 * @param angle The angle in radians
 */

function getDelta(angle) {
  return [Math.cos(angle), Math.sin(angle)];
}

/**
 * getArrow
 * Get the points for a linking line between two points.
 * @description Draw an arrow between two points.
 * @param x0 The x position of the "from" point.
 * @param y0 The y position of the "from" point.
 * @param x1 The x position of the "to" point.
 * @param y1 The y position of the "to" point.
 * @param options Additional options for computing the line.
 * @returns [sx, sy, cx, cy, e1, e2, ae, as, ac]
 * @example
 * const arrow = getArrow(0, 0, 100, 200, {
    bow: 0
    stretch: .5
    stretchMin: 0
    stretchMax: 420
    padStart: 0
    padEnd: 0
    flip: false
    straights: true
 * })
 *
 * const [
 *  startX, startY,
 *  controlX, controlY,
 *  endX, endY,
 *  endAngle,
 *  startAngle,
 *  controlAngle
 *  ] = arrow
 */

function getArrow(x0, y0, x1, y1, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$bow = _options.bow,
      bow = _options$bow === void 0 ? 0 : _options$bow,
      _options$stretch = _options.stretch,
      stretch = _options$stretch === void 0 ? 0.5 : _options$stretch,
      _options$stretchMin = _options.stretchMin,
      stretchMin = _options$stretchMin === void 0 ? 0 : _options$stretchMin,
      _options$stretchMax = _options.stretchMax,
      stretchMax = _options$stretchMax === void 0 ? 420 : _options$stretchMax,
      _options$padStart = _options.padStart,
      padStart = _options$padStart === void 0 ? 0 : _options$padStart,
      _options$padEnd = _options.padEnd,
      padEnd = _options$padEnd === void 0 ? 0 : _options$padEnd,
      _options$flip = _options.flip,
      flip = _options$flip === void 0 ? false : _options$flip,
      _options$straights = _options.straights,
      straights = _options$straights === void 0 ? true : _options$straights;
  var angle = getAngle(x0, y0, x1, y1);
  var dist = getDistance(x0, y0, x1, y1);
  var angliness = getAngliness(x0, y0, x1, y1); // Step 0 ⤜⤏ Should the arrow be straight?

  if (dist < (padStart + padEnd) * 2 || // Too short
  bow === 0 && stretch === 0 || // No bow, no stretch
  straights && [0, 1, Infinity].includes(angliness) // 45 degree angle
  ) {
      // ⤜⤏ Arrow is straight! Just pad start and end points.
      // Padding distances
      var ps = Math.max(0, Math.min(dist - padStart, padStart));
      var pe = Math.max(0, Math.min(dist - ps, padEnd)); // Move start point toward end point

      var _projectPoint = projectPoint(x0, y0, angle, ps),
          _px = _projectPoint[0],
          _py = _projectPoint[1]; // Move end point toward start point


      var _projectPoint2 = projectPoint(x1, y1, angle + Math.PI, pe),
          _px2 = _projectPoint2[0],
          _py2 = _projectPoint2[1]; // Get midpoint between new points


      var _getPointBetween = getPointBetween(_px, _py, _px2, _py2, 0.5),
          _mx = _getPointBetween[0],
          _my = _getPointBetween[1];

      return [_px, _py, _mx, _my, _px2, _py2, angle, angle, angle];
    } // ⤜⤏ Arrow is an arc!
  // Is the arc clockwise or counterclockwise?


  var rot = (getSector(angle) % 2 === 0 ? 1 : -1) * (flip ? -1 : 1); // Calculate how much the line should "bow" away from center

  var arc = bow + mod(dist, [stretchMin, stretchMax], [1, 0], true) * stretch; // Step 1 ⤜⤏ Find padded points.
  // Get midpoint.

  var _getPointBetween2 = getPointBetween(x0, y0, x1, y1, 0.5),
      mx = _getPointBetween2[0],
      my = _getPointBetween2[1]; // Get control point.


  var _getPointBetween3 = getPointBetween(x0, y0, x1, y1, 0.5 - arc),
      cx = _getPointBetween3[0],
      cy = _getPointBetween3[1];

  var _rotatePoint = rotatePoint(cx, cy, mx, my, Math.PI / 2 * rot);

  cx = _rotatePoint[0];
  cy = _rotatePoint[1];
  // Get padded start point.
  var a0 = getAngle(x0, y0, cx, cy);

  var _projectPoint3 = projectPoint(x0, y0, a0, padStart),
      px0 = _projectPoint3[0],
      py0 = _projectPoint3[1]; // Get padded end point.


  var a1 = getAngle(x1, y1, cx, cy);

  var _projectPoint4 = projectPoint(x1, y1, a1, padEnd),
      px1 = _projectPoint4[0],
      py1 = _projectPoint4[1]; // Step 2  ⤜⤏ Find start and end angles.
  // Start angle


  var as = getAngle(cx, cy, x0, y0); // End angle

  var ae = getAngle(cx, cy, x1, y1); // Step 3 ⤜⤏ Find control point for padded points.
  // Get midpoint between padded start / end points.

  var _getPointBetween4 = getPointBetween(px0, py0, px1, py1, 0.5),
      mx1 = _getPointBetween4[0],
      my1 = _getPointBetween4[1]; // Get control point for padded start / end points.


  var _getPointBetween5 = getPointBetween(px0, py0, px1, py1, 0.5 - arc),
      cx1 = _getPointBetween5[0],
      cy1 = _getPointBetween5[1];

  var _rotatePoint2 = rotatePoint(cx1, cy1, mx1, my1, Math.PI / 2 * rot);

  cx1 = _rotatePoint2[0];
  cy1 = _rotatePoint2[1];

  // Finally, average the two control points.
  var _getPointBetween6 = getPointBetween(cx, cy, cx1, cy1, 0.5),
      cx2 = _getPointBetween6[0],
      cy2 = _getPointBetween6[1];

  return [px0, py0, cx2, cy2, px1, py1, ae, as, angle];
}

/**
 * getArrowBetweenBoxes
 * Get the points for a linking line between two boxes.
 * @param x0 The x-axis coordinate of the first box.
 * @param y0 The y-axis coordinate of the first box.
 * @param w0 The width of the first box.
 * @param h0 The height of the first box.
 * @param x1 The x-axis coordinate of the second box.
 * @param y1 The y-axis coordinate of the second box.
 * @param w1 The width of the second box.
 * @param h1 The height of the second box.
 * @param options
 */

function getBoxToBoxArrow(x0, y0, w0, h0, x1, y1, w1, h1, options) {
  if (options === void 0) {
    options = {};
  }

  var i0x, i0y, i1x, i1y;
  var _options = options,
      _options$bow = _options.bow,
      bow = _options$bow === void 0 ? 0 : _options$bow,
      _options$stretch = _options.stretch,
      stretch = _options$stretch === void 0 ? 0.5 : _options$stretch,
      _options$stretchMin = _options.stretchMin,
      stretchMin = _options$stretchMin === void 0 ? 0 : _options$stretchMin,
      _options$stretchMax = _options.stretchMax,
      stretchMax = _options$stretchMax === void 0 ? 420 : _options$stretchMax,
      _options$padStart = _options.padStart,
      padStart = _options$padStart === void 0 ? 0 : _options$padStart,
      _options$padEnd = _options.padEnd,
      padEnd = _options$padEnd === void 0 ? 20 : _options$padEnd,
      _options$flip = _options.flip,
      flip = _options$flip === void 0 ? false : _options$flip,
      _options$straights = _options.straights,
      straights = _options$straights === void 0 ? true : _options$straights;
  var cx0 = x0 + w0 / 2,
      cy0 = y0 + h0 / 2,
      cx1 = x1 + w1 / 2,
      cy1 = y1 + h1 / 2,
      px0 = x0 - padStart,
      py0 = y0 - padStart,
      pw0 = w0 + padStart * 2,
      ph0 = h0 + padStart * 2,
      px1 = x1 - padEnd,
      py1 = y1 - padEnd,
      pw1 = w1 + padEnd * 2,
      ph1 = h1 + padEnd * 2;

  var _getPointBetween = getPointBetween(cx0, cy0, cx1, cy1, 0.5),
      mx = _getPointBetween[0],
      my = _getPointBetween[1];

  var angle = getAngle(cx0, cy0, cx1, cy1);
  var angliness = getAngliness(cx0, cy0, cx1, cy1);
  var minLength = Math.min(pw0, ph0, pw1, ph1) * 2;
  var direct = getDistance(x0, y0, x1, y1);
  var overflow = minLength - direct;
  var isColliding = doRectanglesCollide(px0, py0, pw0, ph0, px1, py1, pw1, ph1);

  if (straights && ([0, 1, Infinity].includes(angliness) || cx0 === cx1 || cy0 === cy1) && !isColliding) {

    var _getSegmentRectangleI = getSegmentRectangleIntersectionPoints(cx0, cy0, cx1, cy1, px0, py0, pw0, ph0);

    var _getSegmentRectangleI2 = _getSegmentRectangleI[0];
    i0x = _getSegmentRectangleI2[0];
    i0y = _getSegmentRectangleI2[1];

    var _getSegmentRectangleI3 = getSegmentRectangleIntersectionPoints(cx0, cy0, cx1, cy1, px1, py1, pw1, ph1);

    var _getSegmentRectangleI4 = _getSegmentRectangleI3[0];
    i1x = _getSegmentRectangleI4[0];
    i1y = _getSegmentRectangleI4[1];
    return [i0x, i0y, mx, my, i1x, i1y, angle, angle + Math.PI, angle];
  } // ⤜⤏ Arrow is an arc!
  // Is the arc clockwise or counterclockwise?


  var rot = (getSector(angle) % 2 === 0 ? 1 : -1) * (flip ? -1 : 1); // Calculate how much the line should "bow" away from center

  var stretchEffect = mod(direct, [stretchMin, stretchMax], [1, 0], true);
  var arc = bow + stretchEffect * stretch / 2; // Step 1 ⤜⤏ Find padded points.
  // Get control point.

  var _getPointBetween2 = getPointBetween(cx0, cy0, cx1, cy1, 0.5 - arc),
      cx = _getPointBetween2[0],
      cy = _getPointBetween2[1];

  var _rotatePoint = rotatePoint(cx, cy, mx, my, Math.PI / 2 * rot);

  cx = _rotatePoint[0];
  cy = _rotatePoint[1];

  if (overflow > 0) {

    var _projectPoint = projectPoint(cx, cy, angle + Math.PI / 2 * -rot, overflow * arc);

    cx = _projectPoint[0];
    cy = _projectPoint[1];
  } // Get padded start point.


  var i0 = getSegmentRoundedRectangleIntersectionPoints(cx0, cy0, cx, cy, px0, py0, pw0, ph0, padStart); // Get padded end point.

  var i1 = getSegmentRoundedRectangleIntersectionPoints(cx1, cy1, cx, cy, px1, py1, pw1, ph1, padEnd); // If we don't have intersections, or if the distance between the
  // intersections is very small, use the backup point-finding stategy

  if (!(i0[0] && i1[0]) || getDistance(i0[0][0], i0[0][1], i1[0][0], i1[0][1]) < padStart + padEnd) {
    // Basically, shoot a ray based on the opposite angle between
    // the two centers, and see where it intersects the rectangles.
    // This part can definitely be improved!
    var _getDelta = getDelta(angle + Math.PI),
        dx = _getDelta[0],
        dy = _getDelta[1];

    var _getRayRectangleInter = getRayRectangleIntersectionPoints(cx0, cy0, dx, dy, px0, py0, pw0, ph0);

    var _getRayRectangleInter2 = _getRayRectangleInter[0];
    i0x = _getRayRectangleInter2[0];
    i0y = _getRayRectangleInter2[1];

    var _getRayRectangleInter3 = getRayRectangleIntersectionPoints(cx1, cy1, dx, dy, px1, py1, pw1, ph1);

    var _getRayRectangleInter4 = _getRayRectangleInter3[0];
    i1x = _getRayRectangleInter4[0];
    i1y = _getRayRectangleInter4[1];
    arc = (bow + stretchEffect * stretch) * -1;
  } else {
    var _i0$ = i0[0];
    i0x = _i0$[0];
    i0y = _i0$[1];
    var _i1$ = i1[0];
    i1x = _i1$[0];
    i1y = _i1$[1];
    arc = bow + stretchEffect * stretch;
  } // Step 3 ⤜⤏ Calculate arrow points using same algorithm as getArrow


  var _getPointBetween3 = getPointBetween(i0x, i0y, i1x, i1y, 0.5),
      mx1 = _getPointBetween3[0],
      my1 = _getPointBetween3[1];

  var _getPointBetween4 = getPointBetween(i0x, i0y, i1x, i1y, 0.5 - arc);

  cx = _getPointBetween4[0];
  cy = _getPointBetween4[1];

  var _rotatePoint2 = rotatePoint(cx, cy, mx1, my1, Math.PI / (2 * rot));

  cx = _rotatePoint2[0];
  cy = _rotatePoint2[1];
  var as = getAngle(cx, cy, i0x, i0y);
  var ae = getAngle(cx, cy, i1x, i1y);
  return [i0x, i0y, cx, cy, i1x, i1y, ae, as, angle];
}

export { getArrow, getBoxToBoxArrow };
//# sourceMappingURL=perfect-arrows.esm.js.map
